.include "tn44def.inc"


.eseg
.org 0x00

Bind_Code:.byte 9

.dseg
.org 0x60

RX_Data:  .byte 16
RX_Summ:  .byte 2

RX_Word:  .byte 1
RX_Time:  .byte 1

Mode:     .byte 1

#define Mode_Bind 0
#define Mode_Send 1
#define Mode_RX   2


#define RX_Port PORTB
#define RX_Pin  PB2

#define R_SREG      R0
#define PWM_Counter R1
#define Chan1       R2
#define Chan2       R3
#define Chan3       R4
#define Chan4       R5
#define Chan5       R6
#define Chan6       R7
#define Chan7       R8
#define Chan8       R9

#define UartStop_RX 2
#define UartStop_TX 8
#define UartF       R10
#define UartF_TX_8  0
#define UartF_RX_8  1
#define UartF_TX_16 2
#define UartF_RX_16 3
#define UartBit     R11
#define UartH       R12
#define UartL       R13



.cseg
.org 0x000  rjmp I_Reset
.org 0x001  rjmp I_Int0
.org 0x009  rjmp I_Tim0_CompA


.include "EEPROM.inc"




I_Reset:

ldi R16,low(RAMEND)
out SPL,R16
ldi R16,high(RAMEND)
out SPH,R16

//PWM outputs
ldi R16,0xFF
out DDRA,R16

//Bind

sbi PORTB,0

nop
nop
nop
nop

ldi R16,(1<<Mode_Send)
sbis PINB,0
ldi R16,(1<<Mode_Bind)
sts Mode,R16



rcall Bind_Load

lds R16,Mode
sbrc R16,Mode_Bind
rcall Bind_Start


// Int0
in R16,MCUCR
ori R16,(0<<ISC01)|(0<<ISC00)
out MCUCR,R16

in R16,GIMSK
ori R16,(1<<INT0)
out GIMSK,R16

//Timer0
ldi R16,(0<<COM0A1)|(0<<COM0A0)|(0<<COM0B1)|(0<<COM0B0)|(1<<WGM01)|(0<<WGM00) //CTC Mode
out TCCR0A,R16
ldi R16,(0<<FOC0A)|(0<<FOC0B)|(0<<WGM02)|(0<<CS02)|(0<<CS01)|(1<<CS00)  //NoPrescaler
out TCCR0B,R16
ldi R16,(0<<OCIE0B)|(1<<OCIE0A)|(0<<TOIE0)  //I_Tim0_CompA
out TIMSK0,R16
ldi R16,200
out OCR0A,R16

rcall Uart_Init

lds R16,Mode
sbrc R16,Mode_Send
rcall TX_Start



sei



MainLoop:
rjmp MainLoop


TX_Start:
  ldi R16,byte1(500000)
  ldi R17,byte2(500000)
  ldi R18,byte3(500000)
  TX_Start_Loop:
    subi R16,1
    sbci R17,0
    sbci R18,0
  brne TX_Start_Loop
  rcall TX_Read_EE
ret

Bind_Start:
  cbi RX_Port,RX_Pin
  sbi RX_Port-1,RX_Pin

  ldi R16,byte1(500000)
  ldi R17,byte2(500000)
  ldi R18,byte3(500000)
  Bind_Start_Loop:
    subi R16,1
    sbci R17,0
    sbci R18,0
  brne Bind_Start_Loop

  cbi RX_Port-1,RX_Pin
  sbi RX_Port,RX_Pin
ret

Bind_Save:
  ldi YL,low(RX_Data)
  ldi YH,high(RX_Data)
  ldi ZL,low(Bind_Code)
  ldi ZH,high(Bind_Code)
  ldi R17,10
  Bind_Save_Loop:
    ld R16,Y+
    rcall EEPROM_Write
    subi ZL,low(-1)
    sbci ZH,high(-1)
    dec R17
  brne Bind_Save_Loop
ret

Bind_Load:
  ldi YL,low(RX_Data)
  ldi YH,high(RX_Data)
  ldi ZL,low(Bind_Code)
  ldi ZH,high(Bind_Code)
  ldi R17,10
  Bind_Load_Loop:
    rcall EEPROM_Read
    st Y+,R16
    subi ZL,low(-1)
    sbci ZH,high(-1)
    dec R17
  brne Bind_Load_Loop
ret

Calc_Chan:
  #define Center 1530
  lds R16,RX_Data+0
  lds R17,RX_Data+1
  subi R16,low(Center)
  sbci R17,high(Center)
  lsr R17
  ror R16
  mov Chan1,R16
  com R16
  mov Chan2,R16
  sbrc R17,0
  clr Chan1
  sbrs R17,0
  clr Chan2

  lds R16,RX_Data+2
  lds R17,RX_Data+3
  subi R16,low(Center)
  sbci R17,high(Center)
  lsr R17
  ror R16
  mov Chan3,R16
  com R16
  mov Chan4,R16
  sbrc R17,0
  clr Chan3
  sbrs R17,0
  clr Chan4

  lds R16,RX_Data+4
  lds R17,RX_Data+5
  subi R16,low(Center-430)
  sbci R17,high(Center-430)
  brsh SkipThrottle
    clr R16
    clr R17
  SkipThrottle:
  lsr R17
  ror R16
  lsr R17
  ror R16
  mov Chan5,R16
  com R16
  mov Chan6,R16

  lds R16,RX_Data+6
  lds R17,RX_Data+7
  subi R16,low(Center)
  sbci R17,high(Center)
  lsr R17
  ror R16
  mov Chan7,R16
  com R16
  mov Chan8,R16
  sbrc R17,0
  clr Chan7
  sbrs R17,0
  clr Chan8

ret



Uart_Init:
  clr UartF

  clr R16
  sts RX_Word,R16
  sts RX_Time,R16
  sts RX_Summ+0,R16
  sts RX_Summ+1,R16

  clr Chan1
  clr Chan2
  clr Chan3
  clr Chan4
  clr Chan5
  clr Chan6
  clr Chan7
  clr Chan8
ret




TX_Read_EE:
  lds ZL,RX_Word
  clr ZH
  subi ZL,low(-RX_Data)
  sbci ZH,high(-RX_Data)
  ld UartL,Z
  rcall TX_8_Start
ret

TX_8_Start:
  ldi R16,(1<<UartF_TX_8)
  mov UartF,R16
  ldi R16,8+UartStop_TX
  mov UartBit,R16

  in R16,GIMSK
  cbr R16,(1<<INT0)
  out GIMSK,R16

  sbi RX_Port,RX_Pin
  sbi RX_Port-1,RX_Pin
ret

TX_8_End:
  cbi RX_Port-1,RX_Pin
  clr UartF
  in R16,GIFR
  cbr R16,(1<<INTF0)
  out GIFR,R16
  in R16,GIMSK
  sbr R16,(1<<INT0)
  out GIMSK,R16

  lds ZL,RX_Word
  inc ZL
  sts RX_Word,ZL

  cpi ZL,11
  brsh TX_8_End_End
    rcall TX_Read_EE
    rjmp TX_8_End_Exit
  TX_8_End_End:
    ldi R16,(1<<Mode_RX)
    sts Mode,R16
  TX_8_End_Exit:
ret

TX_8:
  clr R16
  sts RX_Time,R16

  mov R16,UartBit
  cpi R16,UartStop_TX+8
  brsh TX_8_StartBit
  cpi R16,UartStop_TX
  brlo TX_8_StopBit
    sbrc UartL,0
    sbi RX_Port,RX_Pin
    sbrs UartL,0
    cbi RX_Port,RX_Pin
    lsr UartL
    rjmp TX_8_Skip
  TX_8_StartBit:
    cbi RX_Port,RX_Pin
    rjmp TX_8_Skip
  TX_8_StopBit:
    sbi RX_Port,RX_Pin
  TX_8_Skip:

  dec UartBit
  brne TX_8_NoEnd
    rcall TX_8_End
  TX_8_NoEnd:
ret






RX_8_Start:
  ldi R16,(1<<UartF_RX_8)
  mov UartF,R16
  ldi R16,8+UartStop_RX
  mov UartBit,R16

  in R16,GIMSK
  cbr R16,(1<<INT0)
  out GIMSK,R16

  ldi R16,195
  out TCNT0,R16
ret


RX_8_End:
  clr UartF
  in R16,GIFR
  cbr R16,(1<<INTF0)
  out GIFR,R16
  in R16,GIMSK
  sbr R16,(1<<INT0)
  out GIMSK,R16

  lds ZL,RX_Word
  inc ZL
  sts RX_Word,ZL

  cpi ZL,11
  brsh RX_8_End_Exit
    //Add checksum
    lds R16,RX_Summ+0
    add R16,UartL
    sts RX_Summ+0,R16

    //Save byte
    dec ZL
    clr ZH
    subi ZL,low(-RX_Data)
    sbci ZH,high(-RX_Data)
    st Z,UartL
    lds ZL,RX_Word
    cpi ZL,10
    brlo RX_8_End_Exit
      lds R16,RX_Summ+0
      sub R16,UartL
      sub R16,UartL
      ldi R16,0
      sts RX_Summ+0,R16
      brne RX_8_End_Exit
        rcall Bind_Save
  RX_8_End_Exit:
ret

RX_8:
  clr R16
  sts RX_Time,R16

  mov R16,UartBit
  cpi R16,UartStop_RX+8
  brsh RX_8_Skip
  cpi R16,UartStop_RX
  brlo RX_8_Skip
    ldi R16,0x80
    lsr UartL
    sbic RX_Port-2,RX_Pin
    or UartL,R16
  RX_8_Skip:

  dec UartBit
  brne RX_8_NoEnd
    rcall RX_8_End
  RX_8_NoEnd:
ret










RX_16_Start:
  ldi R16,(1<<UartF_RX_16)
  mov UartF,R16
  ldi R16,16+UartStop_RX
  mov UartBit,R16

  in R16,GIMSK
  cbr R16,(1<<INT0)
  out GIMSK,R16

  ldi R16,195
  out TCNT0,R16
ret


RX_16_End:
  clr UartF
  in R16,GIFR
  cbr R16,(1<<INTF0)
  out GIFR,R16
  in R16,GIMSK
  sbr R16,(1<<INT0)
  out GIMSK,R16

  lds ZL,RX_Word
  inc ZL
  sts RX_Word,ZL

  cpi ZL,10
  brsh RX_16_End_Exit
    cpi ZL,9
    brsh RX_16_End_SkipSave
      //Add checksum
      lds R16,RX_Summ+0
      add R16,UartL
      sts RX_Summ+0,R16
      lds R16,RX_Summ+1
      adc R16,UartH
      sts RX_Summ+1,R16

      //Save word
      dec ZL
      lsl ZL
      clr ZH
      subi ZL,low(-RX_Data)
      sbci ZH,high(-RX_Data)
      st Z+,UartL
      st Z+,UartH
      rjmp RX_16_End_Exit
    RX_16_End_SkipSave:
      lds R16,RX_Summ+0
      sub R16,UartL
      lds R16,RX_Summ+1
      sbc R16,UartH
      ldi R16,0
      sts RX_Summ+0,R16
      sts RX_Summ+1,R16
      brne RX_16_End_Exit
        rcall Calc_Chan
  RX_16_End_Exit:
ret


RX_16:
  clr R16
  sts RX_Time,R16

  mov R16,UartBit
  cpi R16,UartStop_RX+16
  brsh RX_16_Skip
  cpi R16,UartStop_RX
  brlo RX_16_Skip
    ldi R16,0x80
    lsr UartH
    ror UartL
    sbic RX_Port-2,RX_Pin
    or UartH,R16
  RX_16_Skip:

  dec UartBit
  brne RX_16_NoEnd
    rcall RX_16_End
  RX_16_NoEnd:
ret








I_Int0:
  in R_SREG,SREG

  lds R16,Mode
  sbrc R16,Mode_RX
    rcall RX_16_Start
  lds R16,Mode
  sbrc R16,Mode_Bind
    rcall RX_8_Start

  out SREG,R_SREG
reti


I_Tim0_CompA:
  in R_SREG,SREG



  //_____________ UART _________________

  sbrc UartF,UartF_TX_8
  rcall TX_8
  sbrc UartF,UartF_RX_8
  rcall RX_8
  sbrc UartF,UartF_RX_16
  rcall RX_16


  lds R16,RX_Time
  inc R16
  cpi R16,30
  brlo SkipReset
    clr R17
    sts RX_Word,R17
    dec R16
  SkipReset:
  sts RX_Time,R16


  //_____________ PWM _________________


  ldi R16,5
  add PWM_Counter,R16

  clr R16

  cp PWM_Counter,Chan1
  brsh NoChan1
    sbr R16,(1<<0)
  NoChan1:
  
  cp PWM_Counter,Chan2
  brsh NoChan2
    sbr R16,(1<<1)
  NoChan2:

  cp PWM_Counter,Chan3
  brsh NoChan3
    sbr R16,(1<<2)
  NoChan3:

  cp PWM_Counter,Chan4
  brsh NoChan4
    sbr R16,(1<<3)
  NoChan4:

  cp PWM_Counter,Chan5
  brsh NoChan5
    sbr R16,(1<<4)
  NoChan5:

  cp PWM_Counter,Chan6
  brsh NoChan6
    sbr R16,(1<<5)
  NoChan6:

  cp PWM_Counter,Chan7
  brsh NoChan7
    sbr R16,(1<<6)
  NoChan7:

  cp PWM_Counter,Chan8
  brsh NoChan8
    sbr R16,(1<<7)
  NoChan8:

  out PORTA,R16



  out SREG,R_SREG
reti






